@@ -34,6 +34,7 @@ import hubzero.data.db
 import ldap
 import os
 import sys
+import traceback
 
 
 def userExists(username, hubname = ""):
@@ -84,7 +85,7 @@ def _del_jos_users(db, username):
 	data = (username)
 	userID = db.query_selectscalar(sql, data)	
 
-	sql = 'delete from `jos_users` where userid = %s'
+	sql = 'delete from `jos_users` where id = %s'
 	data = (userID)
 	userID = db.query_selectscalar(sql, data)	
 
@@ -105,7 +106,6 @@ def _del_jos_core_acl_aro(db, userID):
 
 
 def _del_jos_core_acl_groups_aro_map(db, aroID):
-	print "aroID = " + str(aroID)
 	sql = "DELETE FROM `jos_core_acl_groups_aro_map` WHERE `aro_id` = %s"
 	data = (aroID)
 	db.query_rowcount(sql, data)
@@ -162,7 +162,7 @@ def _update_jos_users_password(db, userid, password):
 
 def _update_ldap_user_pw(username, uidNumber, password):
 	"""
-	Use raw pw here, we'll hash it
+	Pass raw pw here, we'll hash it
 	"""
 
 	if password != '*':
@@ -170,7 +170,7 @@ def _update_ldap_user_pw(username, uidNumber, password):
 	elif '{MD5}' in password:
 		pw = password
 	else:
-		pw = '{MD5}*' # can user generate a value that will MD5 hash to, we think not '*'?
+		pw = '{MD5}*' # can user generate a value that will MD5 hash to? We think not
 
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
@@ -294,7 +294,7 @@ def add_ldap_user(uid,
 	elif '{MD5}' in password:
 		attrs['userPassword'] = password
 	else:
-		attrs['userPassword'] = '{MD5}*' # can user generate a value that will MD5 hash to, we think not '*'?
+		attrs['userPassword'] = '{MD5}*' # can user generate a value that will MD5 hash to '*'?, we think not 
 
 	attrs['uid'] = uid
 	attrs['uidNumber'] = str(uidNumber)
@@ -319,7 +319,11 @@ def delete_ldap_user(username, hubname = ""):
 	l = ldap.initialize('ldap://localhost')
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 	dn = "uid=" + username + ",ou=users," + hubLDAPBaseDN
-	l.delete_s(dn)
+
+	try:
+		l.delete_s(dn)
+	except ldap.NO_SUCH_OBJECT:
+		pass
 
 
 def addhubuser(hubname, 
@@ -347,6 +351,7 @@ def addhubuser(hubname,
 	email - user email
 	pw - unencrypted password, we'll hash it as necessary using MD5
 	pwdisabled - should the PW be disabled?
+	grouptype - 0 (system), 1 (regular)
 	joomlagidNumber - 18=Registered, 23=Manager, 24=Administrator, 25=Super Administrator
 	gid - linux gid membership
 	gidNumber - linux gidNumber, corresponds to the group with the previous gid
@@ -419,7 +424,6 @@ def addhubuser(hubname,
 		gid = 'users'
 
 	if not skipCreateCMSUser:
-
 		# create group for user named same as username (need group insert here for xprofiles addition below)
 		# Guess if user specifies a specific already created group for the new user and says create a new group for the 
 		# user, we'll override and create the new group with this logic
@@ -440,7 +444,6 @@ def addhubuser(hubname,
 		# create home directory
 		if not skipCreateHome:
 			rc, procStdOut, procStdErr = hubzero.utilities.misc.exShellCommand(["mkdir", homeDir, "--mode", "0750"])
-			print procStdOut
 			if rc:
 				print procStdErr
 
@@ -478,12 +481,12 @@ def delhubuser(username, hubname = ""):
 	data = (username)
 	userID = db.query_selectscalar(sql, data)
 
-	print "deleting user (" + username + ") from database"
+	if uidExists(username):
+		aroID = _del_jos_core_acl_aro(db, userID)
+		if aroID: _del_jos_core_acl_groups_aro_map(db, aroID)
 
-	aroID = _del_jos_core_acl_aro(db, userID)
-	_del_jos_core_acl_groups_aro_map(db, aroID)
-	_del_jos_xprofiles(db, userID)
-	_del_jos_users(db, username)
+		_del_jos_xprofiles(db, userID)
+		_del_jos_users(db, username)
 
 	delete_ldap_user(username)
 
@@ -540,7 +543,6 @@ def uidExists(uid, hubname = ""):
 	"""
 	See if specified uid is in the ldap user ou
 	"""
-
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn", hubname)
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw", hubname)
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn", hubname)	
@@ -582,3 +584,122 @@ def getLDAPMinUIDNumber(hubname = "", minUIDNumber=1, maxUIDNumber=1000):
 			break
 
 	return rv
+
+
+def syncUser(username, hubname = "", syncAttributes = False):
+	"""
+	sync attributes syncs all available non essential account attributes.
+	"""
+	
+	if not hubname:
+		hubname = hubzero.config.webconfig.getDefaultSite()
+
+	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn", hubname)
+	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw", hubname)
+	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn", hubname)	
+
+	dbPW = hubzero.config.webconfig.getWebConfigDBPassword(hubname)
+	dbUserName = hubzero.config.webconfig.getWebConfigDBUsername(hubname)
+	dbName = hubzero.config.webconfig.getWebConfigDBName(hubname)
+	db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
+
+	query = """SELECT u.id AS uidNumber, u.username AS uid, u.name AS cn, p.gidNumber, p.homeDirectory, p.loginShell, 
+	pwd.passhash AS userPassword, pwd.shadowLastChange, pwd.shadowMin, pwd.shadowMax, pwd.shadowWarning, 
+	pwd.shadowInactive, pwd.shadowExpire, pwd.shadowFlag 
+	FROM jos_users AS u 
+	LEFT JOIN jos_users_password AS pwd ON u.id = pwd.user_id 
+	LEFT JOIN jos_xprofiles AS p ON u.id = p.uidNumber 
+	WHERE u.username = %s; 
+	"""
+	data = (username)
+	user = db.query(query, data)
+
+	attrsToSync = {}
+	if syncAttributes:
+
+		attributesToCheck = ['userPassword', 'shadowLastChange', 'shadowMin', 'shadowMax', 'shadowWarning', 'shadowInactive', 'shadowExpire', 'shadowFlag']
+		
+		# set all the attributes
+		for a in attributesToCheck:
+			av = getattr(user[0], a, None)
+			if av:
+				attrsToSync[str(a)] = str(av)
+		
+	else:
+		attrsToSync['userPassword'] = user[0].userPassword
+	
+	if user:
+		add_ldap_user2(user[0].cn,
+			           user[0].uid,
+			           user[0].uidNumber,
+			           user[0].homeDirectory, 
+			           user[0].gidNumber, 
+		               hubname,
+		               attrsToSync)
+	else:
+		print "syncuser cannot find info on username: " + username + " from the jos_users table"
+		
+
+		
+def add_ldap_user2(cn,
+                  uid, 
+                  uidNumber, 
+                  homeDir, 
+                  gidNumber, 
+                  hubname = "",
+                  passedAttributes = {}):
+
+	"""
+	Following ldap attributes are required by the follwing objectclasses
+	# account - uid
+	# posixAccount - cn, uid, uidNumber, gidNumber, homeDirectory
+	# shadowAccount - uid
+
+	All other optional attributes can be passed in the list and we'll grab
+	them. If the attribute is invalid, shame on you
+	"""
+
+	# get ldapManagerUserDN and PW
+	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn", hubname)
+	hubLDAPAcctMgrPW= hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw", hubname)
+	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn", hubname)	
+
+	l = ldap.initialize('ldap://localhost')
+	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+
+	dn = "uid=" + str(uid) + ",ou=users," + hubLDAPBaseDN
+
+	attrs = {}
+	attrs['objectclass'] = ['top', 'account', 'posixAccount', 'shadowAccount']  
+	attrs['cn'] = str(uid)
+	attrs['uid'] = uid
+	attrs['uidNumber'] = str(uidNumber)
+	attrs['homeDirectory'] = homeDir
+	attrs['gidNumber'] = str(gidNumber)
+
+	allAttrs = dict(attrs.items() + passedAttributes.items())
+	ldif = ldap.modlist.addModlist(allAttrs)
+
+	try:
+		l.add_s(dn, ldif)
+	except ldap.ALREADY_EXISTS:
+		# delete then readd
+		delete_ldap_user(uid)
+		l.add_s(dn, ldif)
+		
+		# code below was for an in place update, kept around just in case
+		# get previous attribute list for object
+		#results = l.search_s('ou=users,' + hubLDAPBaseDN, ldap.SCOPE_SUBTREE, '(uidNumber=' + str(uidNumber) + ')', None)
+
+		# second item [1] is the attribute list of the first ldap entry [0] returned by the search
+		# construct modified attribute list 
+		#ldif_mod = ldap.modlist.modifyModlist(results[0][1], dict(attrs.items() + passedAttributes.items()))
+		
+		#try:
+		#	l.modify_s(dn, ldif_mod)
+		#except Exception as e1:
+		#	print "error updating " + uid
+		#	print e1
+		#	traceback.print_exc()
+		
+	
\ No newline at end of file
